UTF8 = "utf-8"

APPLICATION_JSON = "application/json"
TEXT_PLAIN = "text/plain"
TEXT_HTML = "text/html"

NOT_FOUND = "Not Found"

WEBSOCKET_SEND_TYPE = "websocket.send"
WEBSOCKET_ACCEPT_TYPE = "websocket.accept"
WEBSOCKET_CLOSE_TYPE = "websocket.close"
WEBSOCKET_RECEIVE_TYPE = "websocket.receive"
WEBSOCKET_DISCONNECT_TYPE = "websocket.disconnect"
WEBSOCKET_TYPE = "websocket"

SRC_DIRECTORY = "src"
